#include <limits.h>
#include <glib.h>

#include <pan/base/acache.h>
#include <pan/base/pan-i18n.h>
#include <pan/base/pan-glib-extensions.h>
#include <pan/base/base-prefs.h>

char * data_dir = NULL;
char * download_dir = NULL;
char * default_incoming_name_real;
char * default_incoming_name_addr;
int acache_max_megs = 10;
gboolean acache_flush_on_exit = FALSE;
gboolean break_thread_when_subject_changes = FALSE;

static gboolean _quote_chars_impl[UCHAR_MAX];
const gboolean * const _quote_chars = _quote_chars_impl;

void
base_prefs_init (const char    * new_data_dir,
                 const char    * new_download_dir,
                 const char    * new_quote_chars,
                 int             new_acache_max_megs,
                 gboolean        new_acache_flush_on_exit,
                 gboolean        break_thread)
{
	int i;

	replace_gstr (&data_dir, g_strdup(new_data_dir));
	replace_gstr (&download_dir, g_strdup(new_download_dir));
	break_thread_when_subject_changes = break_thread;

	acache_max_megs = new_acache_max_megs;
	acache_flush_on_exit = new_acache_flush_on_exit;
	acache_init ();

	default_incoming_name_real = _("Unknown");
	default_incoming_name_addr = _("unknown@spammesenseless.net");

	/* set _quote_chars */
	for (i=0; i<UCHAR_MAX; ++i)
		_quote_chars_impl[i] = FALSE;
	for (i=0; new_quote_chars!=NULL && new_quote_chars[i]!='\0'; ++i)
		_quote_chars_impl[(guchar)new_quote_chars[i]] = TRUE;
}
