/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999, 2000  Pan Development Team <pan@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * dialog-about.c: About dialog
 */

#include <config.h>
#include <libgnomeui/libgnomeui.h>
#include <pan/base/pan-i18n.h>
#include <pan/dialogs/dialogs.h>
#include <pan/dialogs/pan-about.h>

static gchar * get_credits_code (void);
static gchar * get_credits_users (void);
static gchar * get_credits_translators (void);

/**
 * dialog_about:
 *
 * About window to display program information, version number, and
 * copyright.
 **/
void
dialog_about (GtkWidget * window)
{
	gchar * pch;
	gchar * credits_bugs;
	gchar * credits_i18n;
	gchar * credits_code;
	GtkWidget * w;
	PanAbout * a;

	static GtkWidget * about = NULL;

	w = pan_about_new ("Pan",
	                   VERSION,
	                   _("A newsreader for GNOME"),
	                   _("Copyright (C) 1999-2001, Pan Development Team"),
	                   PAN_ABOUT_LICENSE_GPL,
	                   "http://pan.rebelbase.com/");

	a = PAN_ABOUT(w);
	pan_about_add_author (a, "Charles Kerr", "charles@rebelbase.com",
	                      _("Lead Programmer"));
	pan_about_add_author (a, "Christophe Lambin", "clambin@easynet.be",
	                      _("Programmer"));
	pan_about_add_author (a, "Adam Huffman", "bloch@verdurin.com",
	                      _("Programmer & Documentation"));
	pan_about_add_author (a, "Matt Eagleson", "e.Messiah@superpimp.org",
	                      _("Project Originator, Programmer"));
	pan_about_add_author (a, "Jason Leach", "jleach@ximian.com",
	                      _("Programmer, 0.6 - 0.7"));

	credits_i18n = get_credits_translators ();
	credits_code = get_credits_code ();
	credits_bugs = get_credits_users ();
	pch = g_strdup_printf (_("Many thanks to the following people, whose contributions\n"
	                         "are a large part of why Open Source software development\n"
	                         "is so fun.\n"
                                 "\n"
                                 "Pan %s Translations\n%s\n"
                                 "\n"
                                 "Pan %s Code or Patches\n%s\n"
                                 "\n"
                                 "Pan %s Suggestions and Testing\n%s\n"
                                 "\n"
                                 "If you should have been in this list but you're not,\n"
                                 "send us mail and we'll add you next time.\n"
                                 "\n"
	                         "Contributors to previous versions are listed at\n"
	                         "http://pan.rebelbase.com/CREDITS"),
	                         VERSION, credits_i18n,
	                         VERSION, credits_code,
	                         VERSION, credits_bugs);
	pan_about_add_text (a, _("Thanks"), pch);
	g_free (credits_i18n);
	g_free (credits_code);
	g_free (credits_bugs);
	g_free (pch);

	gtk_signal_connect (GTK_OBJECT(w), "destroy",
			    GTK_SIGNAL_FUNC (gtk_widget_destroyed),
			    &about);

	if (window != NULL)
		gnome_dialog_set_parent (GNOME_DIALOG(w),
					 GTK_WINDOW (window));

	gtk_widget_show (w);
}


static gchar *
get_credits_translators (void)
{
	return g_strdup ("\n\tT. Aihana\n\tDuarte Loreto\n\tChristian Rose\n\tMarcel Telka\n\tReinhout van Schouwen");
}

static gchar *
get_credits_code (void)
{
	return g_strdup ("\n\tGabi Davar\n\tJiri Kubicek\n\tColin Leroy\n\tSam Solon\n\tJeff Stedfast");
}

static gchar *
get_credits_users (void)
{
	return g_strdup ("\n\tErik Anderson\n\tNed Boddie\n\tMike Brodbelt\n\tHaines Brown\n\tMartin Bruchanov\n\tAric S. Campling\n\tBill Cattell\n\tJeff E.\n\tHimanshu J. Gohel \n\tAurelien Gouny\n\tJim Henderson\n\tBill Huang\n\tJohn Levon\n\tMirko Liss\n\tMichael McIntyre\n\tJohn Mellor\n\tSven Neuhaus\n\tMr. P\n\tCraig Orsinger\n\tWilliam Peterson\n\tJean Ribiere\n\tKerstin Rohlfs\n\tJoshua Rosen\n\tJason Rumney\n\tJorge Salinas\n\tFlorian Schmidt\n\tT'aZ\n\tPascal Valois");
}
